from typing import Optional

import streamlit as st

from .app import _get_current_app, App


def get_current_app(app_id: Optional[str] = None, use_cache: bool = False) -> App:
    query_params = st.experimental_get_query_params()

    return _get_current_app(
        app_id=app_id,
        session=st.session_state,
        token=query_params.get("token", [None])[0],
        use_cache=use_cache,
    )



from . import _ResourceGroup

from .capabilities import _HasTeamsMixin, _HasSportingEventsMixin, _HasVideosMixin


class PersonResourceGroup(
    _ResourceGroup, _HasTeamsMixin, _HasSportingEventsMixin, _HasVideosMixin
):
    pass



from pyteamtv.infra.requester import Requester


class TeamTVObject(object):
    def __init__(self, requester: Requester, attributes: dict):
        self._requester = requester
        self.__attributes = attributes

        self._use_attributes(attributes)

    @property
    def metadata(self):
        return self._metadata

    @property
    def is_local(self):
        return self._is_local

    def _use_attributes(self, attributes: dict):
        self._metadata = attributes.get("_metadata", {})

        shared_type = self._metadata.get("source", {}).get("type", {})
        self._is_local = (shared_type is None) or (shared_type == "ResourceGroup")

    def has_privilege(self, action: str) -> bool:
        for privilege, status in self.metadata.get("privilegesV2", {}).items():
            if privilege.endswith(":" + action):
                return status["ok"]
        return False



import os
from typing import Optional

try:
    __PYTEAMTV_SETUP__
except NameError:
    __PYTEAMTV_SETUP__ = False

if not __PYTEAMTV_SETUP__:
    from .api import TeamTVApp, TeamTVUser

    def get_team(name: Optional[str] = None, resource_group_id: Optional[str] = None):
        if "TEAMTV_API_TOKEN" in os.environ:
            app = TeamTVUser(os.environ["TEAMTV_API_TOKEN"])
            return app.get_team(name=name, resource_group_id=resource_group_id)
        else:
            raise Exception("'TEAMTV_API_TOKEN' environment variable is not set.")


__version__ = "0.22.1"



from typing import TypedDict, Union

from .teamtv_object import TeamTVObject


class Observation(TeamTVObject):
    @property
    def observation_id(self) -> str:
        return self._observation_id

    @property
    def start_time(self) -> float:
        return self._start_time

    @property
    def trigger_time(self) -> float:
        return self._trigger_time

    @property
    def end_time(self) -> float:
        return self._end_time

    @property
    def code(self) -> str:
        return self._code

    @property
    def clock_id(self) -> str:
        return self._clock_id

    @property
    def description(self) -> str:
        return self._description

    @property
    def attributes(self) -> dict:
        return self._attributes

    def _use_attributes(self, attributes: dict):
        self._observation_id = attributes["observationId"]
        self._start_time = attributes["startTime"]
        self._trigger_time = attributes["triggerTime"]
        self._end_time = attributes["endTime"]
        self._code = attributes["code"]
        self._attributes = attributes["attributes"] or dict()
        self._description = attributes["description"]
        self._clock_id = attributes["clockId"]

        super()._use_attributes(attributes)


class DictObservation(TypedDict):
    startTime: Union[str, float]
    triggerTime: Union[str, float]
    endTime: Union[str, float]
    code: str
    description: str
    attributes: dict



from datetime import timedelta
from typing import Optional, Callable

from flask import request, session, has_request_context, g
from flask_session import Session

from .app import _get_current_app, App


def init_app(app, app_id: str = None) -> Callable[[], Optional[App]]:
    app.config["SESSION_PERMANENT"] = True
    app.config["SESSION_TYPE"] = "filesystem"
    app.config["SESSION_COOKIE_SAMESITE"] = "None"
    app.config["SESSION_COOKIE_SECURE"] = True
    app.config["SESSION_COOKIE_PATH"] = "/"

    app.config["PERMANENT_SESSION_LIFETIME"] = timedelta(hours=5)

    # The maximum number of items the session stores
    # before it starts deleting some, default 500
    app.config["SESSION_FILE_THRESHOLD"] = 100

    sess = Session()
    sess.init_app(app)

    def get_current_app() -> Optional[App]:
        if not has_request_context():
            return None

        if "teamtv_app" not in g:
            return None

        return g.teamtv_app

    @app.before_request
    def reset_app():
        g.teamtv_app = _get_current_app(
            app_id=app_id, session=session, token=request.args.get("token")
        )

    return get_current_app



from dataclasses import dataclass
from typing import List

from .teamtv_object import TeamTVObject


@dataclass
class File(object):
    filename: str
    size: int


class Exercise(TeamTVObject):
    @property
    def title(self):
        return self._title

    @property
    def exercise_id(self):
        return self._exercise_id

    @property
    def video_id(self):
        return self._video_id

    @property
    def tags(self):
        return self._tags

    def init_video_upload(self, files: List[File]):
        # TODO: make sure we can return a partial video object
        video_id = self._requester.request(
            "POST",
            f"/exercises/{self.exercise_id}/initVideoUpload",
            dict(
                files=[dict(filename=file.filename, size=file.size) for file in files]
            ),
        )
        self._video_id = video_id
        return video_id

    def set_variables(self, variables: dict):
        self._requester.request(
            "POST",
            f"/exercises/{self.exercise_id}/setVariables",
            dict(variables=variables),
        )

    def __repr__(self):
        return f'<Exercise exercise_id={self.exercise_id} title="{self.title}" video_id={self.video_id}>'

    def _use_attributes(self, attributes: dict):
        self._exercise_id = attributes["exerciseId"]
        self._title = attributes["title"]
        self._video_id = attributes.get("videoId")
        self._tags = attributes.get("tags", {})

        super()._use_attributes(attributes)



from .teamtv_object import TeamTVObject


class EventStream(TeamTVObject):
    @property
    def event_stream_id(self):
        return self._event_stream_id

    @property
    def endpoint_urls(self):
        return self._endpoint_urls

    def __repr__(self):
        return f"<EventStream event_stream_id={self.event_stream_id}>"

    def _use_attributes(self, attributes: dict):
        self._event_stream_id = attributes["eventStreamId"]
        self._endpoint_urls = attributes["endpointUrls"]

        super()._use_attributes(attributes)
